
/* Chrysalide - Outil d'analyse de fichiers binaires
 * aput.c - décompilation des instructions manipulant des tableaux (enregistrement)
 *
 * Copyright (C) 2010-2013 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "translate.h"


#include "../../../decomp/expr/array.h"
#include "../../../decomp/expr/assign.h"



/******************************************************************************
*                                                                             *
*  Paramètres  : instr = instruction d'origine à convertir.                   *
*                ctx   = contexte de la phase de décompilation.               *
*                                                                             *
*  Description : Décompile une instruction de type 'aput'.                    *
*                                                                             *
*  Retour      : Instruction mise en place ou NULL.                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GDecInstruction *dalvik_decomp_instr_aput(const GArchInstruction *instr, GDecContext *ctx)
{
    GDecInstruction *result;                /* Instruction à retourner     */
    vmpa_t addr;                            /* Adresse de l'instruction    */
    GArchOperand *operand;                  /* Opérande de l'instruction   */
    GDecInstruction *content;               /* Contenu de cellule visé     */
    GDecInstruction *array;                 /* Tableau accédé              */
    GDecInstruction *index;                 /* Indice de cellule considérée*/
    GDecInstruction *access;                /* Représentation de l'accès   */

    g_arch_instruction_get_location(instr, NULL, NULL, &addr);

    operand = g_arch_instruction_get_operand(instr, 0);
    content = g_dec_context_convert_register(ctx, operand, false, addr);

    operand = g_arch_instruction_get_operand(instr, 1);
    array = g_dec_context_convert_register(ctx, operand, false, addr);

    operand = g_arch_instruction_get_operand(instr, 2);
    index = g_dec_context_convert_register(ctx, operand, false, addr);

    access = g_array_access_new(G_DEC_EXPRESSION(array), G_DEC_EXPRESSION(index));
    result = g_assign_expression_new(G_DEC_EXPRESSION(access), G_DEC_EXPRESSION(content));

    return result;

}
