
/* Chrysalide - Outil d'analyse de fichiers binaires
 * iput.c - décompilation des instructions manipulant des champs d'instance (déchargement)
 *
 * Copyright (C) 2012-2013 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "translate.h"


#include "../../../arch/dalvik/operands/pool.h"
#include "../../../decomp/expr/access.h"
#include "../../../decomp/expr/assign.h"
#include "../../../decomp/expr/pseudo.h"



/******************************************************************************
*                                                                             *
*  Paramètres  : instr = instruction d'origine à convertir.                   *
*                ctx   = contexte de la phase de décompilation.               *
*                                                                             *
*  Description : Décompile une instruction de type 'iput'.                    *
*                                                                             *
*  Retour      : Instruction mise en place ou NULL.                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GDecInstruction *dalvik_decomp_instr_iput(const GArchInstruction *instr, GDecContext *ctx)
{
    GDecInstruction *result;                /* Instruction à retourner     */
    vmpa_t addr;                            /* Adresse de l'instruction    */
    GArchOperand *operand;                  /* Opérande de l'instruction   */
    GDecInstruction *dest;                  /* Registre de destination     */
    GDecInstruction *src;                   /* Registre de l'object        */
    uint32_t index;                         /* Indice dans la table        */
    GDecInstruction *field;                 /* Champ concerné par l'opérat°*/
    GBinVariable *var;                      /* Variable / champ accédé     */
    GDecInstruction *access;                /* Représentation de l'accès   */

    g_arch_instruction_get_location(instr, NULL, NULL, &addr);

    operand = g_arch_instruction_get_operand(instr, 0);
    src = g_dec_context_convert_register(ctx, operand, false, addr);

    operand = g_arch_instruction_get_operand(instr, 1);
    dest = g_dec_context_convert_register(ctx, operand, false, addr);

    operand = g_arch_instruction_get_operand(instr, 2);
    index = g_dalvik_pool_operand_get_index(G_DALVIK_POOL_OPERAND(operand));
    var = get_field_from_dex_pool(G_DEX_FORMAT(g_dec_context_get_format(ctx)), index);

    field = g_pseudo_register_new(PRU_FIXED);
    g_pseudo_register_set_variable(G_PSEUDO_REGISTER(field), var);

    access = g_access_expression_new(G_DEC_EXPRESSION(dest), G_DEC_EXPRESSION(field));
    result = g_assign_expression_new(G_DEC_EXPRESSION(access), G_DEC_EXPRESSION(src));

    return result;

}
